
   /**-------------------------------------------------------------------**
    **                               CLooG                               **
    **-------------------------------------------------------------------**
    **                              cloog.h                              **
    **-------------------------------------------------------------------**
    **                   First version: july 25th 2002                   **
    **-------------------------------------------------------------------**/


/******************************************************************************
 *               CLooG : the Chunky Loop Generator (experimental)             *
 ******************************************************************************
 *                                                                            *
 * Copyright (C) 2001-2005 Cedric Bastoul                                     *
 *                                                                            *
 * This library is free software; you can redistribute it and/or              *
 * modify it under the terms of the GNU Lesser General Public                 *
 * License as published by the Free Software Foundation; either               *
 * version 2.1 of the License, or (at your option) any later version.         *
 *                                                                            *
 * This library is distributed in the hope that it will be useful,            *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
 * Lesser General Public License for more details.                            *
 *                                                                            *
 * You should have received a copy of the GNU Lesser General Public           *
 * License along with this library; if not, write to the Free Software        *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,                         *
 * Boston, MA  02110-1301  USA                                                *
 *                                                                            *
 * CLooG, the Chunky Loop Generator                                           *
 * Written by Cedric Bastoul, Cedric.Bastoul@inria.fr                         *
 *                                                                            *
 ******************************************************************************/

/******************************************************************************
 *  THIS FILE HAS BEEN AUTOMATICALLY GENERATED FROM clooh.h.in BY configure   *
 ******************************************************************************/

#ifndef CLOOG_H
#define CLOOG_H

#include <cloog/version.h>
#include <cloog/int.h>
#include <cloog/matrix.h>
#include <cloog/state.h>
#include <cloog/options.h>
#include <cloog/util.h>
#include <cloog/names.h>
#include <cloog/constraints.h>
#include <cloog/stride.h>
#include <cloog/domain.h>
#include <cloog/statement.h>
#include <cloog/block.h>
#include <cloog/loop.h>
#include <cloog/union_domain.h>
#include <cloog/input.h>
#include <cloog/program.h>
#include <cloog/clast.h>
#include <cloog/pprint.h>

#endif /* !CLOOG_H */
